/* LibTomCrypt, modular cryptographic library -- Tom St Denis
 *
 * LibTomCrypt is a library that provides various cryptographic
 * algorithms in a highly modular and flexible manner.
 *
 * The library is free for all purposes without any express
 * guarantee it works.
 *
 * Tom St Denis, tomstdenis@gmail.com, http://libtomcrypt.org
 *
 * slightly modified by the author of TCTEMP
 */
#include "tomcrypt.h"

/**
  @file hmac_done.c
  HMAC support, terminate stream, Tom St Denis/Dobes Vandermeer
*/

#ifdef HMAC

#define HMAC_BLOCKSIZE pdHash->blocksize

/**
   Terminate an HMAC session
   @param hmac    The HMAC state
   @param out     [out] The destination of the HMAC authentication tag
   @param outlen  [in/out]  The max size and resulting size of the HMAC authentication tag
   @return CRYPT_OK if successful
*/
int hmac_done(hmac_state *hmac, unsigned char *out, unsigned long *outlen)
{
    unsigned long hashsize, i;
    int err;
	const struct ltc_hash_descriptor *pdHash;

    LTC_ARGCHK(hmac  != NULL);
    LTC_ARGCHK(out   != NULL);

    pdHash = hmac->pdHash;

    /* get the hash message digest size */
    hashsize = pdHash->hashsize;

    /* Get the hash of the first HMAC vector plus the data */
    if ((err = pdHash->done(&hmac->md, hmac->isha)) != CRYPT_OK) {
       goto LBL_ERR;
    }

    /* Create the second HMAC vector vector for step (3) */
    for(i=0; i < HMAC_BLOCKSIZE; i++) {
        hmac->buf[i] = hmac->key[i] ^ 0x5C;
    }

    /* Now calculate the "outer" hash for step (5), (6), and (7) */
    if ((err = pdHash->init(&hmac->md)) != CRYPT_OK) {
       goto LBL_ERR;
    }
    if ((err = pdHash->process(&hmac->md, hmac->buf, HMAC_BLOCKSIZE)) != CRYPT_OK) {
       goto LBL_ERR;
    }
    if ((err = pdHash->process(&hmac->md, hmac->isha, hashsize)) != CRYPT_OK) {
       goto LBL_ERR;
    }
    if ((err = pdHash->done(&hmac->md, hmac->buf)) != CRYPT_OK) {
       goto LBL_ERR;
    }

    /* copy to output  */
    for (i = 0; i < hashsize && i < *outlen; i++) {
        out[i] = hmac->buf[i];
    }
    *outlen = i;

    err = CRYPT_OK;
LBL_ERR:
    return err;
}

#endif

/* $Source: /cvs/libtom/libtomcrypt/src/mac/hmac/hmac_done.c,v $ */
/* $Revision: 1.3 $ */
/* $Date: 2005/05/05 14:35:58 $ */
